/*
 * =====================================================================================
 *
 *       Filename:  v4l2_capture.h
 *
 *    Description:  v4l2_capture header file
 *
 *        Version:  1.0
 *
 *         Author:  Alexander Aring, a.aring@phytec.de
 *        Company:  PHYTEC
 *
 * =====================================================================================
 */

#ifndef __V4L2_CAPTURE_H__
#define __V4L2_CAPTURE_H__

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <linux/videodev2.h>

#include "regparse.h"

#define REGISTER_SIZE 2 
#define BUFFER_COUNT 1 
#define DEFAULT_VIDEO_DEVICE "/dev/video0"

#define ERR_IMAGE_AGAIN 0
#define ERR_IMAGE_OK 1
#define ERR_IMAGE_ERROR -1

struct buffer {
	void *start;
	size_t length;
};

/*-----------------------------------------------------------------------------
 *  Open device with filepath dev_name.
 *-----------------------------------------------------------------------------*/
int open_video_device(char *dev_name);

/*-----------------------------------------------------------------------------
 *  Close video device.
 *-----------------------------------------------------------------------------*/
int close_video_device();


/*-----------------------------------------------------------------------------
 *  Initialize video device. Check for caps.
 *-----------------------------------------------------------------------------*/
int init_video_device(char *dev_name);

/*-----------------------------------------------------------------------------
 *  Get maximum of framesize for a pixel format. This is a value by referent with
 *  width and height.
 *-----------------------------------------------------------------------------*/
int get_max_framesize(__u32 *width, __u32 *height,
		__u32 pixel_format);

/*-----------------------------------------------------------------------------
 *  Capture one image from a camera with specifig width, height and pixel_format.
 *	The function pointer argument has data image and his imagesize as arguments.
 *-----------------------------------------------------------------------------*/
int capture_image(unsigned int timeout,
		unsigned int count,
		__u32 width,
		__u32 height,
		__u32 pixel_format,
		int process_frame(const void *image, __u32 imagesize));

/*-----------------------------------------------------------------------------
 *  Get a value from a register camera.
 *-----------------------------------------------------------------------------*/
int get_cam_reg(__u32 address, __u64 reg, __u64 *data);

/*-----------------------------------------------------------------------------
 *  Set a value to a register.
 *-----------------------------------------------------------------------------*/
int set_cam_reg(__u32 address, __u64 reg, __u64 data);


#endif /* __V4L2_CAPTURE_H__ */

