/*
 * Copyright (C) 2012 Michael Olbrich <m.olbrich@pengutonix.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */

#include "cpuusage.h"

#include <QtCore/QFile>
#include <QtCore/QTextStream>
#include <QtCore/QStringList>
#include <QtCore/QDebug>

CpuUsage::CpuUsage(QObject *parent) :
	QObject(parent),
	m_interval(1000),
	m_core(0),
	m_data(4, 0),
	m_old(4, 0)
{
	getValues();
	m_id = startTimer(m_interval);
}

CpuUsage::~CpuUsage()
{
}

int CpuUsage::interval() const
{
	return m_interval;
}

void CpuUsage::setInterval(int interval)
{
	if (interval == m_interval)
		return;
	m_interval = interval;
	killTimer(m_id);
	m_id = startTimer(m_interval);
	emit intervalChanged(interval);
}

int CpuUsage::core() const
{
	return m_core;
}

void CpuUsage::setCore(int core)
{
	if (core == m_core)
		return;
	m_core = core;
	emit coreChanged(core);
	getValues();
}

int CpuUsage::user() const
{
	return m_data[0];
}

int CpuUsage::nice() const
{
	return m_data[1];
}

int CpuUsage::system() const
{
	return m_data[2];
}

int CpuUsage::idle() const
{
	return m_data[3];
}

int CpuUsage::all() const
{
	return 100 - m_data[3];
}

void CpuUsage::timerEvent(QTimerEvent *)
{
	getValues();
}

void CpuUsage::getValues()
{
	QString cpu = "cpu" + QString::number(core());

	QFile file("/proc/stat");
	if (!file.open(QIODevice::ReadOnly))
		return;
	QTextStream stream(&file);

	m_data.fill(0);

	do {
		QString line = stream.readLine();
		const QStringList list = line.trimmed().split(" ",  QString::SkipEmptyParts);
		if ((list.count() < 6) || list[0] != cpu)
			continue;

		m_now = 0;
		for (int i = 0; i < list.length()-1; ++i) {
			int value = list[i+1].toInt();
			m_now += value;
			if (i < 4) {
				m_data[i] = (value - m_old[i])*1000/m_interval;
				m_old[i] = value;
			}
		}
		break;
	} while(!stream.atEnd());
	emit userChanged(user());
	emit niceChanged(nice());
	emit systemChanged(system());
	emit idleChanged(idle());
	emit allChanged(all());
}

