/*
 * Copyright (C) 2012 Michael Olbrich <m.olbrich@pengutonix.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */

#ifndef MO_APP_LIST_H
#define MO_APP_LIST_H

#include <moapp.h>

#include <QtCore/QAbstractListModel>
#include <QtCore/QFileSystemWatcher>
#include <QtCore/QList>
#include <QtCore/QStringList>
#include <QtDeclarative/QDeclarativeParserStatus>

class MoAppList : public QAbstractListModel, public QDeclarativeParserStatus
{
	Q_OBJECT
	Q_INTERFACES(QDeclarativeParserStatus)

	Q_PROPERTY(QString category READ category WRITE setCategory NOTIFY categoryChanged)
	Q_PROPERTY(QStringList searchDirs READ searchDirs WRITE setSearchDirs NOTIFY searchDirsChanged)
	Q_PROPERTY(bool listMenus READ listMenus WRITE setListMenus NOTIFY listMenusChanged)
	Q_PROPERTY(int count READ count NOTIFY countChanged)

	friend class MoApp;
public:
	MoAppList(QObject *parent = 0);
	~MoAppList();

	enum Roles {
		NameRole        = Qt::UserRole+1,
		IconRole        = Qt::UserRole+2,
		FileNameRole    = Qt::UserRole+3,
		ModuleRole      = Qt::UserRole+4,
		ElementRole     = Qt::UserRole+5,
		MenuRole        = Qt::UserRole+6,
		LocaleAwareRole = Qt::UserRole+7,
		KeyRole         = Qt::UserRole+101,
		AppRole         = Qt::UserRole+102,
	};

	int rowCount(const QModelIndex &parent) const;
	QVariant data(const QModelIndex &index, int role) const;

	QString category() const;
	void setCategory(const QString &category);

	QStringList searchDirs() const;
	void setSearchDirs(const QStringList &searchDirs);

	bool listMenus() const;
	void setListMenus(bool listMenus);

	int count() const;

	virtual void classBegin();
	virtual void componentComplete();

public slots:
	MoApp *get(int index) const;
	QString key(int index) const;

signals:
	void categoryChanged(const QString &category);
	void searchDirsChanged(const QStringList &searchDirs);
	void listMenusChanged(bool listMenus);
	void update(const QString &key, const QString &type, const QString &value);
	void countChanged(int count);

private slots:
	void reload();
	void setLocale(const QString &locale);

private:
	QVariant get(int index, const QString &key) const;

private:
	QString m_category;
	bool m_listMenus;
	QString m_locale;
	QFileSystemWatcher m_searchDirs;
	QMap<QString, MoApp*> m_apps;
	QStringList m_currentApps;
};

#endif // MO_APP_LIST_H

